/*
* lper.c
* persistent Lua
* Luiz Henrique de Figueiredo <lhf@tecgraf.puc-rio.br>
* 17 May 2007 23:30:04
* based on an original idea by Andr de Leiradella <leiradella@bigfoot.com>
* This code is hereby placed in the public domain.
*/

#include <stdio.h>
#include <stdlib.h>

#include "lpsm.h"

#include "lua.h"
#include "lper.h"

#define LPER_DAT	"lper.dat"
#define LPER_LOG	"lper.log"
#define LPER_AGE	"lper.age"

static void *lper_alloc(void *data, void *p, size_t oldsize, size_t size)
{
 if (size==0)
 {
  lpsm_free(p);
  return NULL;
 }
 else
  return lpsm_realloc(p,size);
}

static int panic (lua_State *L) {
  fprintf(stderr, "PANIC: unprotected error in call to Lua API (%s)\n",
                   lua_tostring(L, -1));
  exit(EXIT_FAILURE);
  return 0;
}

lua_State *lper_open(void)
{
 void **r=lpsm_init(LPER_DAT,LPER_LOG);
 lua_State *L;
 if (r==NULL) return NULL;
 L=*r;
 if (L==NULL) *r=L=lua_newstate(lper_alloc,NULL);
 if (L==NULL) return NULL;
 lua_atpanic(L,&panic);
 lua_settop(L,0);
 return L;
}

void lper_close(lua_State *L)
{
 lua_settop(L,0);
 lua_gc(L,LUA_GCCOLLECT,0);
 lpsm_checkpoint(0.0,PSMSYNC_SYNC);
 lpsm_shutdown();
}

void lper_openlibs(lua_State *L)
{
 lua_getfield(L,LUA_REGISTRYINDEX,LPER_AGE);
 printf("LPER\t%s %d\n",LPER_AGE,lua_tointeger(L,-1));
 if (lua_isnil(L,-1)) luaL_openlibs(L);
 lua_pushnumber(L,lua_tonumber(L,-1)+1);
 lua_setfield(L,LUA_REGISTRYINDEX,LPER_AGE);
 lua_pop(L,1);
}
